@@ -129,112 +129,152 @@ class Action(object):
     
     sort_index = 90
     """
+    
     Predefined sort_index values are:
     
+    ===== =================================
     value action
-    ----- -----------
+    ===== =================================
     20    :class:`Insert <InsertRow>`
     30    :class:`Delete <DeleteSelected>`
     50    :class:`Print <lino.mixins.printable.BasePrintAction>`
     60    :attr:`Duplicate <lino.mixins.duplicable.Duplicable.duplicate_row>`
-    90    default for all custom row actions created using `@dd.action`
+    90    default for all custom row actions created using :func:`@dd.action <action>`
+    ===== =================================
     
     """
-    opens_a_slave = False
+    
     label = None
+    """
+    The text to appear on the button.
+    """
     
-    #~ ruled = False 
-    #~ """
-    #~ Whether this action is ruled by workflows.
-    #~ """
-    actor = None
     help_text = None
-    #~ debug = False
-    name = None
-    url_action_name = None
-    #~ inheritable = True
-    key = None
-    callable_from = None
-    default_format = 'html'
-    readonly = True
-    hide_top_toolbar = False
-    hide_navigator = False
-    opens_a_window = False
-    show_in_bbar = True
     """
-    Whether this action should be displayed as a button in the bottom toolbar and the context menu.
-    """
-    
-    show_in_workflow = False
-    """
-    Whether this action should be displayed 
-    among the :meth:`workflow_buttons <lino.core.actors.Actor.workflow_buttons>`.
+    A help text that shortly explains what this action does.
+    ExtJS uses this as tooltip text.
     """
     
     auto_save = True
     """
     What to do when this action is being called while the user is on a dirty record.
+    
     - `False` means: forget any changes in current record and run the action.
     - `True` means: save any changes in current record before running the action.
     - `None` means: ask the user.
     """
-    #~ can_view = perms.always
     
-    #~ required_user_groups = None
-    #~ required_user_level = None
-    #~ required_states = None
     required = {}
     """
-    A dict with conditions to specify permissions.
-    #~ If this is set, the action is available only on rows 
-    #~ that meet the specified conditions.
-    
+    A dict with permission requirements.
+    See :func:`lino.core.perms.make_permission_handler`.
     """
     
-    #~ owned_only = False
-    #~ """
-    #~ If this is `True` 
-    #~ (and if :attr:`lino.core.actors.Actor.workflow_owner_field` is set),
-    #~ the action will be available only on rows owned by the requesting user.
-    #~ """
+    name = None
+    """
+    Internally used to store the name of this action within the Actor's namespace.
+    """
+    
+    url_action_name = None
+    """
+    """
+    
+    actor = None
+    """
+    Internally used to store the :class:`lino.core.actors.Actor` 
+    who owns this action.
+    """
+    
+    key = None
+    """
+    The hotkey. Currently not used.
+    """
+    
+    callable_from = None
+    """
+    Either `None`(default) or a tuple of class 
+    objects (subclasses of :class:`Action`).
+    If specified, this action is available only within a window 
+    that has been opened by one of the given actions.
+    """
+    
+    default_format = 'html'
+    """
+    Used internally.
+    """
+    
+    readonly = True
+    """
+    Whether this action possibly modifies data *in the given object*.
+    This means that :class:`InsertRow` is a `readonly` action.
+    """
+    
+    opens_a_window = False
+    """
+    Used internally to say whether this action opens a window.
+    """
+    
+    hide_top_toolbar = False
+    """
+    Used internally if :attr:`opens_a_window` to say whether the window has a top toolbar.
+    """
+    
+    hide_navigator = False
+    """
+    Used internally if :attr:`opens_a_window` to say whether the window has a navigator.
+    """
+    
+    show_in_bbar = True
+    """
+    Used internally.
+    Whether this action should be displayed as a button in the bottom toolbar and the context menu.
+    """
+    
+    show_in_workflow = False
+    """
+    Used internally.
+    Whether this action should be displayed 
+    as the :meth:`workflow_buttons <lino.core.actors.Actor.workflow_buttons>`.
+    """
+    
+    custom_handler = False
+    """
+    Whether this action is implemented as Javascript function call.
+    (...)
+    """
     
     
-    
-    
-    #~ def __init__(self,actor=None,name=None,label=None,**kw):
-    #~ def __init__(self,name=None,label=None,url_action_name=None,**kw):
-    #~ def __init__(self,label=None,**kw):
-    def __init__(self,label=None,url_action_name=None,required={},**kw):
-        #~ self.actor = actor # actor who offers this action
-        #~ if actor is not None:
-            #~ self.actor = actor # actor who offers this action
-            #~ if actor.hide_top_toolbar:
-                #~ self.hide_top_toolbar = True
-        
-        if url_action_name is not None:
-            if not isinstance(url_action_name,basestring):
-                raise Exception("%s name %r is not a string" % (self.__class__,url_action_name))
-            self.url_action_name = url_action_name
-        #~ if name is None:
-            #~ if self.name is None:
-                #~ self.name = self.__class__.__name__ 
-        #~ else:
-            #~ self.name = name 
-        #~ if not isinstance(self.name,basestring):
-            #~ raise Exception("%s name %r is not a string" % (self.__class__,self.name))
-        if label is None:
-            label = self.label or self.url_action_name 
-        self.label = label
-        assert self.callable_from is None or isinstance(
-            self.callable_from,(tuple,type)), "%s" % self
+    #~ def __init__(self,label=None,url_action_name=None,required={},**kw):
+    def __init__(self,label=None,**kw):
+        """
+        The first argument is the optional `label`,
+        other arguments should be specified as keywords and can be 
+        any of the existing class attributes.
+        """
+        #~ if url_action_name is not None:
+            #~ if not isinstance(url_action_name,basestring):
+                #~ raise Exception("%s name %r is not a string" % (self.__class__,url_action_name))
+            #~ self.url_action_name = url_action_name
+        if label is not None:
+            self.label = label
+            
+        #~ if label is None:
+            #~ label = self.label or self.url_action_name 
         for k,v in kw.items():
             if not hasattr(self,k):
                 raise Exception("Invalid keyword %s" % k)
             setattr(self,k,v)
-        self.set_required(**required)
+        self.set_required()
+        #~ self.set_required(**required)
+        assert self.callable_from is None or isinstance(
+            self.callable_from,(tuple,type)), "%s" % self
 
         
     def set_required(self,**kw):
+        """
+        Override existing permission requirements.
+        Arguments: see :func:`lino.core.perms.make_permission_handler`.
+        """
         #~ logger.info("20120628 set_required %s(%s)",self,kw)
         new = dict()
         new.update(self.required)
@@ -242,8 +282,9 @@ class Action(object):
         self.required = new
         #~ if isinstance(self,StateAction):
         if self.required.has_key('states'):
-            self.show_in_bbar = False
             self.show_in_workflow = True
+            self.custom_handler = True
+            self.show_in_bbar = False
         else:
             self.show_in_workflow = False
             self.show_in_bbar = True
@@ -268,6 +309,8 @@ class Action(object):
             raise Exception("%s tried to attach action %s of %s" % (actor,name,self.actor))
         self.name = name
         self.actor = actor
+        if self.label is None:
+            self.label = name
         if actor.hide_top_toolbar:
             self.hide_top_toolbar = True
         if self.help_text is None and self is actor.default_action:
@@ -309,7 +352,6 @@ class Action(object):
         Derived Action classes may override this to add vetos.
         E.g. DispatchAction is not available for a User with empty partner.
         """
-        #~ logger.info("20120622 Action.get_action_permission")
         return self.allow(user,obj,state)
         
     #~ def run(self,elem,ar,**kw):
@@ -353,7 +395,7 @@ class RowAction(Action):
 
 
 class StateAction(RowAction):
-    
+    ajax = True
     def __init__(self,actor,target_state,**kw):
         self.target_state = target_state
         kw.update(label=getattr(target_state,'action_label',target_state.text))
@@ -372,15 +414,14 @@ class StateAction(RowAction):
         super(StateAction,self).__init__(**kw)
         #~ print 20120709, self, self.show_in_workflow
         
-    #~ def get_action_permission(self,user,obj,state):
-        #~ if state and not state.get_state_permission(self,user,obj):
-            #~ return False
-        #~ return self.allow(user,obj,state)
-        
     def run(self,row,ar,**kw):
-        old = row.state
+        state_field_name = self.actor.workflow_state_field.attname
+        assert isinstance(state_field_name,basestring)
+        #~ old = row.state
+        old = getattr(row,state_field_name)
         row.before_state_change(ar,kw,old,self.target_state)
-        row.state = self.target_state
+        #~ row.state = self.target_state
+        setattr(row,state_field_name,self.target_state)
         row.save()
         row.after_state_change(ar,kw,old,self.target_state)
         return ar.ui.success_response(**kw)
@@ -403,9 +444,6 @@ class GridEdit(TableAction):
     callable_from = tuple()
     url_action_name = 'grid'
     
-    #~ def __init__(self,*args,**kw):
-        #~ TableAction.__init__(self,*args,**kw)
-        
     def attach_to_actor(self,actor,name):
         #~ self.label = actor.button_label or actor.label
         self.label = actor.label
@@ -445,19 +483,19 @@ class InsertRow(TableAction):
     The new row will be actually created only when this 
     window gets submitted.
     """
+    label = _("New")
     show_in_workflow = False
     opens_a_window = True
+    hide_navigator = True
     sort_index = 20
     hide_top_toolbar = True
-    readonly = False
+    #~ readonly = False # see blog/2012/0726
     required = dict(user_level='user')
     callable_from = (GridEdit,ShowDetailAction)
     url_action_name = 'insert'
     #~ label = _("Insert")
-    label = _("New")
     key = INSERT # (ctrl=True)
     #~ needs_selection = False
-    hide_navigator = True
     
     def get_action_title(self,rr):
         return _("Insert into %s") % force_unicode(rr.get_title())
@@ -465,6 +503,11 @@ class InsertRow(TableAction):
     def get_window_layout(self):
         return self.actor.insert_layout or self.actor.detail_layout
 
+    def get_action_permission(self,user,obj,state):
+        # see blog/2012/0726
+        if user.profile.readonly: 
+            return False
+        return super(InsertRow,self).get_action_permission(user,obj,state)
 
 
 
@@ -479,7 +522,6 @@ class DuplicateRow(RowAction):
     label = _("Duplicate")
 
 
-#~ class ShowEmptyTable(InsertRow):
 class ShowEmptyTable(ShowDetailAction):
     callable_from = tuple()
     url_action_name = 'show' 
@@ -530,12 +572,13 @@ class DeleteSelected(RowAction):
     
         
 class SubmitDetail(RowAction):
+    label = _("Save")
     auto_save = False
     show_in_workflow = False
+    #~ show_in_bbar = True
     readonly = False
     required = dict(user_level='user')
     #~ url_action_name = 'SubmitDetail'
-    label = _("Save")
     callable_from = (ShowDetailAction,)
     
 class SubmitInsert(SubmitDetail):
@@ -545,22 +588,19 @@ class SubmitInsert(SubmitDetail):
     callable_from = (InsertRow,)
 
 
-"""
-"General actions" don't need to know their actor, so we can have 
-the same instance for all actors.
-"""
-#~ VIEW = ViewAction(sort_index=1)
-#~ CREATE = SubmitInsert(sort_index=1)
-#~ UPDATE = SubmitDetail(sort_index=1)
-#~ DELETE = DeleteSelected(sort_index=5)
-
-
-
 
 
 class ActionRequest(object):
     """
-    Deserves more documentation.
+    Holds information about an indivitual web request and provides methods like
+
+    - :meth:`get_user <lino.core.actions.ActionRequest.get_user>`
+    - :meth:`confirm <lino.core.actions.ActionRequest.confirm>`
+    - :meth:`success_response <lino.ui.base.UI.success_response>`
+    - :meth:`error_response <lino.ui.base.UI.error_response>`
+    - :meth:`spawn <lino.core.actions.ActionRequest.spawn>`
+
+    
     """
     create_kw = None
     renderer = None
@@ -604,11 +644,6 @@ class ActionRequest(object):
                     self.param_values.define(k,v)
                 
         
-    #~ def confirm(self,step,*messages):
-        #~ if self.request.REQUEST.get(ext_requests.URL_PARAM_ACTION_STEP,None) == str(step):
-            #~ return
-        #~ raise ConfirmationRequired(step,messages)
-
     def href_to(self,*args,**kw):
         return self.renderer.href_to(self,*args,**kw)
         
@@ -617,6 +652,7 @@ class ActionRequest(object):
             #~ kw.update(param_values=self.ui.parse_params(self.ah,request))
         kw.update(user=request.user)
         kw.update(subst_user=request.subst_user)
+        kw.update(requesting_panel=request.requesting_panel)
         #~ if settings.LINO.user_model:
             #~ username = rqdata.get(ext_requests.URL_PARAM_SUBST_USER,None)
             #~ if username:
@@ -624,6 +660,7 @@ class ActionRequest(object):
                     #~ kw.update(subst_user=settings.LINO.user_model.objects.get(username=username))
                 #~ except settings.LINO.user_model.DoesNotExist, e:
                     #~ pass
+        #~ logger.info("20120723 ActionRequest.parse_req() --> %s",kw)
         return kw
       
     def setup(self,
@@ -631,8 +668,10 @@ class ActionRequest(object):
             subst_user=None,
             param_values={},
             known_values=None,
+            requesting_panel=None,
             renderer=None,
             **kw):
+        self.requesting_panel = requesting_panel
         self.user = user
         if renderer is not None:
             self.renderer = renderer
@@ -653,7 +692,17 @@ class ActionRequest(object):
         
     def confirm(self,*messages):
         """
-        Calling this from an Action's :meth:`Action.run`
+        Calling this from an Action's :meth:`Action.run` method will
+        interrupt the execution, send the specified message(s) back to 
+        the user, waiting for confirmation before continuing.
+        
+        Note that this is implemented without any server sessions 
+        and cookies. While this system is genial, it has one drawback 
+        which you should be aware of: the code execution does not 
+        *continue* after the call to `confirm` but starts again at the 
+        beginning (with the difference that the client this time calls it with 
+        an internal `step` parameter that tells Lino that this `confirm()` 
+        has been answered and should no longer raise stop execution.
         """
         assert len(messages) > 0 and messages[0], "At least one non-empty message required"
         self.step += 1
@@ -697,6 +746,12 @@ class ActionRequest(object):
         #~ return s.encode('us-ascii','replace')
         
     def get_user(self):
+        """
+        Return the :class:`User <lino.modlib.users.models.User>` 
+        instance of the user who issued the request.
+        If the authenticated user is acting as somebody else, 
+        return that :class:`User <lino.modlib.users.models.User>` instance.
+        """
         return self.subst_user or self.user
         
     def get_action_title(self):
@@ -708,19 +763,12 @@ class ActionRequest(object):
     def render_to_dict(self):
         return self.action.render_to_dict(self)
         
-    #~ def row2dict(self,row,d):
-        #~ # overridden in extjs.ext_requests.ViewReportRequest
-        #~ return self.report.row2dict(row,d)
-
     def get_request_url(self,*args,**kw):
         return self.ui.get_request_url(self,*args,**kw)
 
     def get_status(self,ui,**kw):
         if self.actor.parameters:
-            #~ pv = kw.setdefault('param_values',{})
-            #~ kw.update(param_values=self.ah.store.pv2list(self.param_values))
             kw.update(param_values=self.ah.store.pv2dict(ui,self.param_values))
-            #~ kw[ext_requests.URL_PARAM_PARAM_VALUES] = self.ah.store.pv2list(self.param_values)
         bp = kw.setdefault('base_params',{})
         if self.subst_user is not None:
             #~ bp[ext_requests.URL_PARAM_SUBST_USER] = self.subst_user.username
@@ -733,15 +781,14 @@ class ActionRequest(object):
         Create a new ActionRequest, taking default values from this one.
         """
         kw.setdefault('user',self.user)
+        kw.setdefault('subst_user',self.subst_user)
+        kw.setdefault('requesting_panel',self.requesting_panel)
         kw.setdefault('renderer',self.renderer)
         #~ kw.setdefault('request',self.request) 
         # removed 20120702 because i don't want to inherit quick_search from spawning request
         # and because i couldn't remember why 'request' was passed to the spawned request.
         if actor is None:
             actor = self.actor
-        #~ kw.update(request=self.request)
-        #~ return ViewReportRequest(None,rh,rpt.default_action,**kw)
-        #~ return self.__class__(self.ui,actor,**kw)
         return self.ui.request(actor,**kw)
         
     def absolute_uri(self,*args,**kw):
@@ -751,7 +798,16 @@ class ActionRequest(object):
         
 
 def action(*args,**kw):
+    """
+    Decorator to define custom actions.
+    Same signature as :meth:`Action.__init__`.
+    In practice you'll possibly use:
+    :attr:`label <Action.label>`,
+    :attr:`help_text <Action.help_text>` and
+    :attr:`required <Action.required>`
+    """
     def decorator(fn):
+        kw.setdefault('custom_handler',True)
         a = RowAction(*args,**kw)
         #~ a.run = curry(fn,a)
         a.run = fn
